# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fasshim import gravatar_url


class PkgdbProcessor(BaseProcessor):
    __name__ = "Pkgdb"
    __description__ = "the Fedora Package DB"
    __link__ = "https://admin.fedoraproject.org/pkgdb"
    __docs__ = "https://fedorahosted.org/packagedb/"
    __obj__ = "Package ACL Updates"
    __icon__ = ("https://apps.fedoraproject.org/packages/"
               "images/icons/package_128x128.png")

    def subtitle(self, msg, **config):
        if 'pkgdb.acl.update' in msg['topic']:
            tmpl = self._(
                u"{agent} changed {user}'s '{acl}' permission " +
                "on {package} ({branch}) to '{status}'"
            )
            user = msg['msg']['username']
            status = msg['msg']['status']
            package = msg['msg']['package_listing']['package']['name']
            acl = msg['msg']['acl']
            agent = msg['msg']['agent']
            branch = msg['msg']['package_listing']['collection']['branchname']
            return tmpl.format(
                agent=agent, acl=acl,
                user=user, status=status,
                package=package, branch=branch)
        elif 'pkgdb.branch.clone' in msg['topic']:
            tmpl = self._(u"{agent} branched {package} {branch} from {master}")
            agent = msg['msg']['agent']
            package = msg['msg']['package']
            branch = msg['msg']['branch']
            master = msg['msg']['master']
            return tmpl.format(agent=agent, package=package,
                               branch=branch, master=master)
        elif 'pkgdb.package.update' in msg['topic']:
            tmpl = self._(u"{agent} made some updates to {package}")
            agent = msg['msg']['agent']
            try:
                package = msg['msg']['package_listing']['package']['name']
            except KeyError:
                package = msg['msg']['package']

            return tmpl.format(agent=agent, package=package)
        elif 'pkgdb.critpath.update' in msg['topic']:
            tmpl = self._(
                u"{agent} altered the critpath status for some packages")
            agent = msg['msg']['agent']
            return tmpl.format(agent=agent)
        elif 'pkgdb.package.new' in msg['topic']:
            tmpl = self._(
                u"{agent} added a new package '{package}' ({branch})")
            agent = msg['msg']['agent']
            package = msg['msg']['package_listing']['package']['name']
            branch = msg['msg']['package_listing']['collection']['branchname']
            return tmpl.format(agent=agent, package=package, branch=branch)
        elif 'pkgdb.acl.request.toggle' in msg['topic']:
            tmpl = self._(
                u"{agent} has {action} '{acl}' on {package} ({branch})"
            )
            package = msg['msg']['package_listing']['package']['name']
            acl = msg['msg']['acl']
            agent = msg['msg']['agent']
            branch = msg['msg']['package_listing']['collection']['branchname']
            action = msg['msg']['acl_action']
            return tmpl.format(
                agent=agent, acl=acl, action=action,
                package=package, branch=branch)
        elif 'pkgdb.package.retire' in msg['topic']:
            tmpl = self._(
                u"{agent} {action} {package} ({branch})!"
            )
            package = msg['msg']['package_listing']['package']['name']
            action = msg['msg']['retirement']
            agent = msg['msg']['agent']
            branch = msg['msg']['package_listing']['collection']['branchname']
            return tmpl.format(
                agent=agent, action=action,
                package=package, branch=branch)
        elif 'pkgdb.owner.update' in msg['topic']:
            tmpl = self._(
                u"{agent} changed owner of {package} ({branch}) to '{owner}'")

            # Owners got renamed to points of contact in packagedb2
            try:
                owner = msg['msg']['package_listing']['point_of_contact']
            except KeyError:
                owner = msg['msg']['package_listing']['owner']

            package = msg['msg']['package_listing']['package']['name']
            agent = msg['msg']['agent']
            branch = msg['msg']['package_listing']['collection']['branchname']
            return tmpl.format(
                agent=agent,
                owner=owner,
                package=package,
                branch=branch)
        elif 'pkgdb.acl.user.remove' in msg['topic']:
            tmpl = self._(
                u"{agent} removed {user} from {package} ({branches})")
            package = msg['msg']['package_listings'][0]['package']['name']
            user = msg['msg']['username']
            agent = msg['msg']['agent']
            branches = ", ".join([
                p['collection']['branchname']
                for p in msg['msg']['package_listings']
            ])
            return tmpl.format(
                agent=agent,
                user=user,
                package=package,
                branches=branches)
        elif 'pkgdb.branch.start' in msg['topic']:
            tmpl = self._(
                u"{agent} started a branch of {tobranch} from {frombranch}")
            agent = msg['msg']['agent']
            frombranch = msg['msg']['collection_from']['branchname']
            tobranch = msg['msg']['collection_to']['branchname']
            return tmpl.format(
                agent=agent,
                frombranch=frombranch,
                tobranch=tobranch,
            )
        elif 'pkgdb.branch.complete' in msg['topic']:
            tmpl = self._(
                u"{agent}'s branch of {tobranch} from {frombranch} completed")
            agent = msg['msg']['agent']
            frombranch = msg['msg']['collection_from']['branchname']
            tobranch = msg['msg']['collection_to']['branchname']
            return tmpl.format(
                agent=agent,
                frombranch=frombranch,
                tobranch=tobranch,
            )
        elif 'pkgdb.collection.new' in msg['topic']:
            tmpl = self._(
                u"{agent} created a new collection for {name} {version}")
            agent = msg['msg']['agent']
            name = msg['msg']['collection']['name']
            version = msg['msg']['collection']['version']
            return tmpl.format(
                agent=agent,
                name=name,
                version=version,
            )
        elif 'pkgdb.collection.update' in msg['topic']:
            tmpl = self._(
                u"{agent} updated the following fields of the "
                "{name} {version} collection: {fields}")
            agent = msg['msg']['agent']
            name = msg['msg']['collection']['name']
            version = msg['msg']['collection']['version']
            fields = ", ".join(msg['msg']['fields'])
            return tmpl.format(
                agent=agent,
                name=name,
                version=version,
                fields=fields,
            )
        else:
            raise NotImplementedError("%r" % msg)

    def secondary_icon(self, msg, **config):
        user = None

        try:
            user = msg['msg']['agent']
        except KeyError:
            pass

        try:
            user = msg['msg']['username']
        except KeyError:
            pass

        if not user:
            return ""

        return gravatar_url(username=user)

    def usernames(self, msg, **config):
        users = set()

        try:
            users.add(msg['msg']['agent'])
        except KeyError:
            pass

        try:
            users.add(msg['msg']['package_listing']['point_of_contact'])
        except KeyError:
            pass

        try:
            users.add(msg['msg']['package_listing']['owner'])
        except KeyError:
            pass

        try:
            users.add(msg['msg']['username'])
        except KeyError:
            pass

        if 'orphan' in users:
            users.remove('orphan')

        return users

    def objects(self, msg, **config):
        objs = set()

        _msg = msg['msg']

        if 'pkgdb.acl.update' in msg['topic']:
            objs.add('{package}/acls/{branch}/{acl}/{user}'.format(
                package=_msg['package_listing']['package']['name'],
                branch=_msg['package_listing']['collection']['branchname'],
                acl=_msg['acl'],
                user=_msg['username']
            ))
        elif 'pkgdb.package.new' in msg['topic']:
            objs.add('{package}/create'.format(
                package=_msg['package_listing']['package']['name'],
            ))
        elif 'pkgdb.acl.request.toggle' in msg['topic']:
            objs.add('{package}/acls/{branch}/{acl}/{user}'.format(
                package=_msg['package_listing']['package']['name'],
                branch=_msg['package_listing']['collection']['branchname'],
                acl=_msg['acl'],
                user=_msg['agent']
            ))
        elif 'pkgdb.package.retire' in msg['topic']:
            objs.add('{package}/retire'.format(
                package=_msg['package_listing']['package']['name'],
            ))
        elif 'pkgdb.acl.user.remove' in msg['topic']:
            objs.add('{package}/remove/{user}'.format(
                package=_msg['package_listings'][0]['package']['name'],
                user=_msg['username'],
            ))
        elif 'pkgdb.owner.update' in msg['topic']:
            objs.add('{package}/owner/{branch}'.format(
                package=_msg['package_listing']['package']['name'],
                branch=_msg['package_listing']['collection']['branchname'],
            ))
        elif 'pkgdb.package.update' in msg['topic']:
            try:
                package = _msg['package_listing']['package']['name']
            except KeyError:
                package = _msg['package']
            objs.add('{package}/update'.format(package=package))
        elif 'pkgdb.branch.clone' in msg['topic']:
            objs.add('{package}/branch'.format(package=_msg['package']))

        return objs

    def packages(self, msg, **config):
        packages = set()

        try:
            packages.add(msg['msg']['package_listing']['package']['name'])
        except KeyError:
            pass

        try:
            for package in msg['msg']['package_listings']:
                packages.add(package['package']['name'])
        except KeyError:
            pass

        try:
            if isinstance(msg['msg']['package'], basestring):
                packages.add(msg['msg']['package'])
        except KeyError:
            pass

        return packages

    def link(self, msg, **config):
        tmpl = "https://admin.fedoraproject.org/pkgdb/acls/name/{package}"

        if any(map(msg['topic'].__contains__, [
            'pkgdb.acl.update',
            'pkgdb.acl.request.toggle',
            'pkgdb.owner.update',
            'pkgdb.package.retire',
            'pkgdb.package.new',
        ])):
            return tmpl.format(
                package=msg['msg']['package_listing']['package']['name']
            )

        if any(map(msg['topic'].__contains__, [
            'pkgdb.acl.user.remove',
        ])):
            return tmpl.format(
                package=msg['msg']['package_listings'][0]['package']['name']
            )

        if any(map(msg['topic'].__contains__, [
            'pkgdb.package.update',
            'pkgdb.branch.clone',
        ])):
            try:
                package = msg['msg']['package_listing']['package']['name']
            except KeyError:
                package = msg['msg']['package']
            return tmpl.format(package=package)

        return ""



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
from fedmsg_meta_fedora_infrastructure import BaseProcessor

from fasshim import gravatar_url


class KojiProcessor(BaseProcessor):
    __name__ = "buildsys"
    __description__ = "the Fedora build system"
    __link__ = "http://koji.fedoraproject.org/koji"
    __docs__ = "https://fedoraproject.org/wiki/Using_the_Koji_build_system"
    __obj__ = "Koji Builds"
    __icon__ = ("http://fedoraproject.org/w/uploads/2/20/"
                "Artwork_DesignService_koji-icon-48.png")

    def subtitle(self, msg, **config):
        inst = msg['msg'].get('instance', 'primary')
        if inst == 'primary':
            inst = ''
        else:
            inst = ' (%s)' % inst

        if 'buildsys.tag' in msg['topic']:
            tmpl = self._(
                "{owner}'s {name}-{version}-{release} tagged "
                "into {tag} by {user}{inst}"
            )
            return tmpl.format(inst=inst, **msg['msg'])
        elif 'buildsys.untag' in msg['topic']:
            tmpl = self._(
                "{owner}'s {name}-{version}-{release} untagged "
                "from {tag} by {user}{inst}"
            )
            return tmpl.format(inst=inst, **msg['msg'])
        elif 'buildsys.repo.init' in msg['topic']:
            tmpl = self._('Repo initialized:  {tag}{inst}')
            return tmpl.format(inst=inst, **msg['msg'])
        elif 'buildsys.repo.done' in msg['topic']:
            tmpl = self._('Repo done:  {tag}{inst}')
            return tmpl.format(inst=inst, **msg['msg'])
        elif 'buildsys.package.list.change' in msg['topic']:
            tmpl = self._(
                "Package list change for {package}:  '{tag}'{inst}")
            return tmpl.format(inst=inst, **msg['msg'])
        elif 'buildsys.build.state.change' in msg['topic']:
            templates = [
                self._("{owner}'s {name}-{version}-{release} "
                       "started building{inst}"),
                self._("{owner}'s {name}-{version}-{release} "
                       "completed{inst}"),
                self._("{owner}'s {name}-{version}-{release} "
                       "was deleted{inst}"),
                self._("{owner}'s {name}-{version}-{release} "
                       "failed to build{inst}"),
                self._("{owner}'s {name}-{version}-{release} "
                       "was cancelled{inst}"),
            ]
            tmpl = templates[msg['msg']['new']]

            # If there was no owner of the build, chop off the prefix.
            if not msg['msg']['owner']:
                tmpl = tmpl[len("{owner}'s "):]

            return tmpl.format(inst=inst, **msg['msg'])
        elif 'buildsys.task.state.change' in msg['topic']:
            templates = {
                'OPEN': self._(
                    "{owner}'s scratch build of {srpm} started{inst}"),
                'FAILED': self._(
                    "{owner}'s scratch build of {srpm} failed{inst}"),
                'CLOSED': self._(
                    "{owner}'s scratch build of {srpm} completed{inst}"),
                'CANCELED': self._(
                    "{owner}'s scratch build of {srpm} "
                    "was cancelled{inst}"),
            }
            default = self._(
                "{owner}'s scratch build of {srpm} changed{inst}")
            tmpl = templates.get(msg['msg']['new'], default)

            # If there was no owner of the build, chop off the prefix.
            if not msg['msg']['owner']:
                tmpl = tmpl[len("{owner}'s "):]

            return tmpl.format(inst=inst, **msg['msg'])
        else:
            raise NotImplementedError("%r" % msg)

    def secondary_icon(self, msg, **config):
        owner = msg['msg'].get('owner')

        if owner:
            return gravatar_url(owner)

        return self.__icon__

    def usernames(self, msg, **config):
        if 'buildsys.tag' in msg['topic']:
            return set([
                msg['msg']['owner'],
                msg['msg']['user'],
            ])
        elif 'buildsys.untag' in msg['topic']:
            return set([
                msg['msg']['owner'],
                msg['msg']['user'],
            ])
        elif 'buildsys.repo.init' in msg['topic']:
            return set()
        elif 'buildsys.repo.done' in msg['topic']:
            return set()
        elif 'buildsys.package.list.change' in msg['topic']:
            return set()
        elif 'buildsys.build.state.change' in msg['topic']:
            if msg['msg']['owner']:
                return set([
                    msg['msg']['owner'],
                ])

            # Sometimes there is no owner
            return set()
        elif 'buildsys.task.state.change' in msg['topic']:
            if msg['msg']['owner']:
                return set([
                    msg['msg']['owner'],
                ])

            # Sometimes there is no owner
            return set()
        else:
            raise NotImplementedError("%r" % msg)

    def packages(self, msg, **config):
        if 'buildsys.tag' in msg['topic']:
            return set([msg['msg']['name']])
        elif 'buildsys.untag' in msg['topic']:
            return set([msg['msg']['name']])
        elif 'buildsys.repo.init' in msg['topic']:
            return set([])
        elif 'buildsys.repo.done' in msg['topic']:
            return set([])
        elif 'buildsys.package.list.change' in msg['topic']:
            return set([msg['msg']['package']])
        elif 'buildsys.build.state.change' in msg['topic']:
            return set([msg['msg']['name']])
        elif 'buildsys.task.state.change' in msg['topic']:
            # We can't *really* associate scratch builds with a package,
            # honestly.
            return set([])
        else:
            raise NotImplementedError("%r" % msg)

    def link(self, msg, **config):

        instance = msg['msg'].get('instance', 'primary')
        if instance == 'primary':
            base = "http://koji.fedoraproject.org/koji"
        elif instance == 'ppc':
            base = "http://ppc.koji.fedoraproject.org/koji"
        elif instance == 's390':
            base = "http://s390.koji.fedoraproject.org/koji"
        elif instance == 'arm':
            base = "http://arm.koji.fedoraproject.org/koji"
        else:
            raise NotImplementedError("Unhandled instance")

        if 'buildsys.tag' in msg['topic']:
            return base + "/taginfo?tagID=%i" % (
                msg['msg']['tag_id'])
        elif 'buildsys.untag' in msg['topic']:
            return base + "/taginfo?tagID=%i" % (
                msg['msg']['tag_id'])
        elif 'buildsys.repo.init' in msg['topic']:
            return base + "/taginfo?tagID=%i" % (
                msg['msg']['tag_id'])
        elif 'buildsys.repo.done' in msg['topic']:
            return base + "/taginfo?tagID=%i" % (
                msg['msg']['tag_id'])
        elif 'buildsys.build.state.change' in msg['topic']:
            return base + "/buildinfo?buildID=%i" \
                % (msg['msg']['build_id'])
        elif 'buildsys.task.state.change' in msg['topic']:
            return base + "/taskinfo?taskID=%i" \
                % (msg['msg']['id'])
        elif 'buildsys.package.list.change' in msg['topic']:
            return None
        else:
            raise NotImplementedError("%r" % msg)

    def objects(self, msg, **config):
        instance = msg['msg'].get('instance', 'primary')

        if 'buildsys.tag' in msg['topic']:
            return set([
                '/'.join([
                    instance,
                    'builds',
                    msg['msg']['name'],
                    msg['msg']['version'],
                    msg['msg']['release'],
                ]),
                '/'.join([
                    instance,
                    'tags',
                    msg['msg']['tag'],
                ]),
            ])
        elif 'buildsys.untag' in msg['topic']:
            return set([
                '/'.join([
                    instance,
                    'builds',
                    msg['msg']['name'],
                    msg['msg']['version'],
                    msg['msg']['release'],
                ]),
                '/'.join([
                    instance,
                    'tags',
                    msg['msg']['tag'],
                ]),
            ])
        elif 'buildsys.build.state.change' in msg['topic']:
            return set(['/'.join([
                instance,
                'builds',
                msg['msg']['name'],
                msg['msg']['version'],
                msg['msg']['release'],
            ])])
        elif 'buildsys.task.state.change' in msg['topic']:
            return set(['/'.join([
                instance,
                'scratch_builds',
                msg['msg']['srpm'],
            ])])
        elif 'buildsys.repo.init' in msg['topic']:
            return set(['/'.join([
                instance,
                'repos',
                msg['msg']['tag'],
            ])])
        elif 'buildsys.repo.done' in msg['topic']:
            return set(['/'.join([
                instance,
                'repos',
                msg['msg']['tag'],
            ])])
        elif 'buildsys.package.list.change' in msg['topic']:
            return set(['/'.join([
                instance,
                'tags',
                msg['msg']['tag'],
            ])])
        else:
            raise NotImplementedError("%r" % msg)



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for ansible messages """

import unittest

from fedmsg.tests.test_meta import Base

from common import add_doc


class TestAnsiblePlaybookStart(Base):
    """ These messages are published when an admin begins an ansible
    playbook run.  We use that system to manage the servers that
    run fedoraproject.org.
    """

    expected_title = "ansible.playbook.start"
    expected_subti = ('ralph started an ansible run of '
                      'playbooks/groups/badges-backend.yml')
    expected_link = ("http://infrastructure.fedoraproject.org/cgit/"
                     "ansible.git/tree/playbooks/groups/badges-backend.yml")
    expected_icon = "https://apps.fedoraproject.org/img/icons/ansible.png"
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['playbooks/groups/badges-backend.yml'])
    msg = {
        "username": "root",
        "i": 1,
        "timestamp": 1375753735.32427,
        "topic": "org.fedoraproject.prod.ansible.playbook.start",
        "msg": {
            "playbook_checksum": False,
            "extra_vars": {},
            "userid": "ralph",
            "playbook": "/srv/web/infra/ansible/playbooks/groups/"
            "badges-backend.yml",
            "check": False,
            "inventory": "/srv/web/infra/ansible/inventory/"
        }
    }


class TestAnsiblePlaybookComplete(Base):
    """ These messages are published when an admin finishes an ansible
    playbook run.  We use that system to manage the servers that
    run fedoraproject.org.  Here's an example with a playbook that has been
    checked into our SCM repo.
    """
    expected_title = "ansible.playbook.complete"
    expected_subti = ("ralph's playbooks/groups/badges-backend.yml playbook"
                      " run completed")
    expected_link = ("http://infrastructure.fedoraproject.org/cgit/"
                     "ansible.git/tree/playbooks/groups/badges-backend.yml")
    expected_icon = "https://apps.fedoraproject.org/img/icons/ansible.png"
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'playbooks/groups/badges-backend.yml',
        'inventory/badges-backend01.phx2.fedoraproject.org',
        'inventory/badges-backend01.stg.phx2.fedoraproject.org',
    ])
    msg = {
        "username": "root",
        "i": 2,
        "timestamp": 1375753955.771203,
        "topic": "org.fedoraproject.prod.ansible.playbook.complete",
        "msg": {
            "userid": "ralph",
            "playbook": "/srv/web/infra/ansible/playbooks/groups/"
            "badges-backend.yml",
            "results": {
                "badges-backend01.phx2.fedoraproject.org": {
                    "failures": 0,
                    "skipped": 9,
                    "ok": 56,
                    "changed": 1,
                    "unreachable": 0
                },
                "badges-backend01.stg.phx2.fedoraproject.org": {
                    "failures": 0,
                    "skipped": 9,
                    "ok": 56,
                    "changed": 1,
                    "unreachable": 0
                }
            }
        }
    }


class TestAnsiblePlaybookCompleteNotCheckIn(Base):
    """ These messages are published when an admin finishes an ansible
    playbook run.  We use that system to manage the servers that
    run fedoraproject.org.  Here's an example with a playbook that is not
    actually checked into our SCM repo.
    """

    expected_title = "ansible.playbook.complete"
    expected_subti = "ralph's badges-backend.yml playbook run completed"
    expected_link = None
    expected_icon = "https://apps.fedoraproject.org/img/icons/ansible.png"
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'uncontrolled-playbooks/badges-backend.yml',
        'inventory/badges-backend01.phx2.fedoraproject.org',
        'inventory/badges-backend01.stg.phx2.fedoraproject.org',
    ])
    msg = {
        "username": "root",
        "i": 2,
        "timestamp": 1375753955.771203,
        "topic": "org.fedoraproject.prod.ansible.playbook.complete",
        "msg": {
            "userid": "ralph",
            "playbook": "/home/fedora/ralph/ansible/playbooks/groups/"
            "badges-backend.yml",
            "results": {
                "badges-backend01.phx2.fedoraproject.org": {
                    "failures": 0,
                    "skipped": 9,
                    "ok": 56,
                    "changed": 1,
                    "unreachable": 0
                },
                "badges-backend01.stg.phx2.fedoraproject.org": {
                    "failures": 0,
                    "skipped": 9,
                    "ok": 56,
                    "changed": 1,
                    "unreachable": 0
                }
            }
        }
    }


add_doc(locals())


if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for badges messages """

import unittest

from fedmsg.tests.test_meta import Base

from common import add_doc


class TestNewBadge(Base):
    """ These messages are published when `Open Badges
    <https://fedoraproject.org/wiki/Open_Badges>`_ are awarded to Fedora
    Contributors by the fedbadges backend.
    """

    expected_title = "fedbadges.badge.award"
    expected_subti = 'ralph has been awarded the ' + \
        '"Something on your mind" badge'
    expected_link = "https://badges.fedoraproject.org/user/ralph"
    expected_icon = "http://example.com/image.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['something-on-your-mind'])
    msg = {
        u'i': 1,
        u'msg': {
            u'badge': {
                u'description': u'You have commented on 2 or more bodhi '
                'updates.',
                u'creator': u'ralph',
                u'discussion': u'http://github.com/fedora-infra/badges/'
                'pull/SOME_NUMBER',
                u'issuer_id': u'fedora-project',
                u'trigger': {
                    u'topic': u'org.fedoraproject.stg.bodhi.update.comment'
                },
                u'image_url': u'http://example.com/image.png',
                u'criteria': {
                    u'datanommer': {
                        u'filter': {
                            u'topics': [
                                u'{topic}'
                            ]
                        },
                        u'operation': u'count',
                        u'condition': {
                            u'greater than or equal to': 2
                        }
                    }
                },
                u'name': u'Something on your mind'
            },
            u'user': {
                u'badges_user_id': 1,
                u'username': u'ralph',
            }
        },
        u'topic': u'org.fedoraproject.stg.fedbadges.badge.award',
        u'username': u'fedmsg',
        u'timestamp': 1371498303.125771,
    }


class TestRankAdvance(Base):
    """ When a user's rank on the leaderboard of the `Fedora Badges
    <https://badges.fedoraproject.org>`_ system increases, this message gets
    published.
    """
    expected_title = "fedbadges.person.rank.advance"
    expected_subti = "ralph moved to position 1500 on the badges leaderboard"
    expected_link = "https://badges.fedoraproject.org/user/ralph"
    expected_icon = "https://apps.fedoraproject.org/img/icons/badges.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set([])
    msg = {
        "username": "fedmsg",
        "i": 4,
        "timestamp": 1377701575.214381,
        "topic": "org.fedoraproject.prod.fedbadges.person.rank.advance",
        "msg": {
            "old_rank": None,
            "person": {
                "website": None,
                "bio": None,
                "rank": 1500,
                "email": "ralph@fedoraproject.org",
                "nickname": "ralph",
                "id": 1600
            }
        }
    }


class TestFirstLogin(Base):
    """ When a user logs in to the `Fedora Badges
    <https://badges.fedoraproject.org>`_ site for the very first time, we
    publish a message like this one.
    """
    expected_title = "fedbadges.person.login.first"
    expected_subti = "ralph logged in to badges.fedoraproject.org " + \
        "for the first time"
    expected_link = "https://badges.fedoraproject.org/user/ralph"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set([])
    msg = {
        "i": 2,
        "msg": {
            "user": {
                "badges_user_id": 2,
                "username": "ralph"
            }
        },
        "msg_id": "2013-be88d409-cdd7-47f0-9edd-87088f8505d2",
        "source_name": "datanommer",
        "source_version": "0.6.0",
        "timestamp": 1382804277.0,
        "topic": "org.fedoraproject.prod.fedbadges.person.login.first"
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
import socket
hostname = socket.gethostname()

config = dict(
    # This is a dict of possible addresses from which fedmsg can send
    # messages.  fedmsg.init(...) requires that a 'name' argument be passed
    # to it which corresponds with one of the keys in this dict.
    endpoints={
        # These are here so your local box can listen to the upstream
        # infrastructure's bus.  Cool, right?  :)
        "fedora-infrastructure": [
            "tcp://hub.fedoraproject.org:9940",
            #"tcp://stg.fedoraproject.org:9940",
        ],

        # For other, more 'normal' services, fedmsg will try to guess the
        # name of it's calling module to determine which endpoint definition
        # to use.  This can be overridden by explicitly providing the name in
        # the initial call to fedmsg.init(...).
        "bodhi.%s" % hostname: ["tcp://127.0.0.1:3001"],
        "fas.%s" % hostname: ["tcp://127.0.0.1:3002"],
        "fedoratagger.%s" % hostname: ["tcp://127.0.0.1:3003"],
        "mediawiki.%s" % hostname: ["tcp://127.0.0.1:3004"],
        "pkgdb.%s" % hostname: ["tcp://127.0.0.1:3005"],
        "busmon.%s" % hostname: ["tcp://127.0.0.1:3006"],
    },
)



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for pkgdb messages """

import unittest

from fedmsg.tests.test_meta import Base

from common import add_doc


class TestPkgdbACLUpdate(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes these messages when an ACL changes on a package.
    """

    expected_title = "pkgdb.acl.update"
    expected_subti = ("ralph changed ralph's 'watchbugzilla' permission on "
                      "python-sh (EL-6) to 'Awaiting Review'")
    expected_link = "https://admin.fedoraproject.org/pkgdb/acls/name/python-sh"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set(['python-sh'])
    expected_usernames = set(['ralph', 'grover'])
    expected_objects = set(['python-sh/acls/EL-6/watchbugzilla/ralph'])
    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1357576703.125622,
        "topic": "org.fedoraproject.stg.pkgdb.acl.update",
        "msg": {
            "status": "Awaiting Review",
            "username": "ralph",
            "package_listing": {
                "point_of_contact": "grover",
                "package": {
                    "upstreamurl": None,
                    "name": "python-sh",
                    "description": None,
                    "reviewurl": None,
                    "summary": "Python module to simplify calling "
                    "shell commands"
                },
                "qacontact": None,
                "collection": {
                    "pendingurltemplate": None,
                    "name": "Fedora EPEL",
                    "publishurltemplate": None,
                    "version": "6",
                    "disttag": ".el6",
                    "branchname": "EL-6"
                },
                "specfile": None
            },
            "agent": "ralph",
            "acl": "watchbugzilla"
        }
    }


class TestPkgdbPackageNew(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes this message when a new package is added to the DB.  This
    typically happens near the end of the Package Review Process as a
    result of a `SCM Admin Request
    <http://fedoraproject.org/wiki/Package_SCM_admin_requests>`_.
    """
    expected_title = "pkgdb.package.new"
    expected_subti = "ralph added a new package 'php-zmq' (devel)"
    expected_link = "https://admin.fedoraproject.org/pkgdb/acls/name/php-zmq"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set(['php-zmq'])
    expected_usernames = set(['ralph', 'lmacken'])
    expected_objects = set(['php-zmq/create'])
    msg = {
        "username": "apache",
        "i": 3,
        "timestamp": 1357580533.5999,
        "topic": "org.fedoraproject.stg.pkgdb.package.new",
        "msg": {
            "package_listing": {
                "point_of_contact": "lmacken",
                "package": {
                    "upstreamurl": None,
                    "name": "php-zmq",
                    "description": None,
                    "reviewurl": None,
                    "summary": "PHP 0MQ/zmq/zeromq extension"
                },
                "qacontact": None,
                "collection": {
                    "pendingurltemplate": None,
                    "name": "Fedora",
                    "publishurltemplate": None,
                    "version": "19",
                    "disttag": ".f19",
                    "branchname": "devel"
                },
                "specfile": None
            },
            "agent": "ralph"
        }
    }


class TestPkgdbOwnerUpdate(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes this message when a package gets an new owner.  (It is
    also published when a package is orphaned; the 'owner' field will have
    the string 'orphan' as its value.)
    """
    expected_title = "pkgdb.owner.update"
    expected_subti = "ralph changed owner of php-zmq (EL-6) to 'orphan'"
    expected_link = "https://admin.fedoraproject.org/pkgdb/acls/name/php-zmq"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set(['php-zmq'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['php-zmq/owner/EL-6'])
    msg = {
        "username": "apache",
        "i": 3,
        "timestamp": 1357580533.5999,
        "topic": "org.fedoraproject.stg.pkgdb.owner.update",
        "msg": {
            "package_listing": {
                "point_of_contact": "orphan",
                "package": {
                    "upstreamurl": None,
                    "name": "php-zmq",
                    "description": None,
                    "reviewurl": None,
                    "summary": "PHP 0MQ/zmq/zeromq extension"
                },
                "qacontact": None,
                "collection": {
                    "pendingurltemplate": None,
                    "name": "Fedora EPEL",
                    "publishurltemplate": None,
                    "version": "6",
                    "disttag": ".el6",
                    "branchname": "EL-6"
                },
                "specfile": None
            },
            "agent": "ralph"
        }
    }


class TestLegacyPkgdbACLRequestToggle(Base):
    """ The old Fedora Package DB1 published this message when an ACL request
    was toggled on a package.
    """
    expected_title = "pkgdb.acl.request.toggle"
    expected_subti = "ralph has requested 'commit' on php-zmq (EL-6)"
    expected_link = "https://admin.fedoraproject.org/pkgdb/acls/name/php-zmq"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set(['php-zmq'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['php-zmq/acls/EL-6/commit/ralph'])
    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1357581512.006664,
        "topic": "org.fedoraproject.stg.pkgdb.acl.request.toggle",
        "msg": {
            "acl_action": "requested",
            "package_listing": {
                "owner": "orphan",
                "package": {
                    "upstreamurl": None,
                    "name": "php-zmq",
                    "description": None,
                    "reviewurl": None,
                    "summary": "PHP 0MQ/zmq/zeromq extension"
                },
                "qacontact": None,
                "collection": {
                    "pendingurltemplate": None,
                    "name": "Fedora EPEL",
                    "publishurltemplate": None,
                    "version": "6",
                    "disttag": ".el6",
                    "branchname": "EL-6"
                },
                "specfile": None
            },
            "acl_status": "Awaiting Review",
            "agent": "ralph",
            "acl": "commit"
        }
    }


class TestLegacyPkgdbPackageUpdate(Base):
    """ Test old school messages. """
    expected_title = "pkgdb.package.update"
    expected_subti = "ralph made some updates to php-zmq"
    expected_link = "https://admin.fedoraproject.org/pkgdb/acls/name/php-zmq"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                              "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                              "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set(['php-zmq'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['php-zmq/update'])
    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1357581512.006664,
        "topic": "org.fedoraproject.stg.pkgdb.package.update",
        "msg": {
            "acl_action": "requested",
            "package": "php-zmq",
            "agent": "ralph",
        },
    }


class TestPkgdbPackageUpdate(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes this message when metadata for a package is updated.
    """
    expected_title = "pkgdb.package.update"
    expected_subti = "ralph made some updates to guake"
    expected_link = "https://admin.fedoraproject.org/pkgdb/acls/name/guake"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set(['guake'])
    expected_usernames = set(['ralph', 'pingou'])
    expected_objects = set(['guake/update'])
    msg = {
        "username": "apache",
        "i": 144,
        "timestamp": 1379605523.496933,
        "msg_id": "2013-c131fb95-0a2e-4426-95c3-09766e017d29",
        "topic": "org.fedoraproject.dev.pkgdb.package.update",
        "msg": {
            "status": "Approved",
            "package_listing": {
                "package": {
                    "status": "Approved",
                    "upstream_url": "http://guake.org",
                    "name": "guake",
                    "creation_date": 1379619917.0,
                    "summary": "Top down terminal for GNOME",
                    "review_url": "https://bugzilla.redhat.com/450189"
                },
                "collection": {
                    "pendingurltemplate": None,
                    "publishurltemplate": None,
                    "branchname": "F-18",
                    "name": "Fedora",
                    "version": "18"
                },
                "point_of_contact": "pingou"
            },
            "prev_status": "Retired",
            "agent": "ralph",
            "package_name": "guake"
        }
    }


class TestPkgdbBranchClone(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages on this topic when a new branch is cloned for a
    package.
    """
    expected_title = "pkgdb.branch.clone"
    expected_subti = "ralph branched php-zmq f18 from devel"
    expected_link = "https://admin.fedoraproject.org/pkgdb/acls/name/php-zmq"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set(['php-zmq'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['php-zmq/branch'])

    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1357581512.006664,
        "topic": "org.fedoraproject.stg.pkgdb.branch.clone",
        "msg": {
            "package": "php-zmq",
            "branch": "f18",
            "master": "devel",
            "agent": "ralph",
        },
    }


class TestPkgdbCritpathUpdate(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages on this topic when the critical path status of a
    package changes (when it is either added, or removed from the critical
    path).  For example:
    """
    expected_title = "pkgdb.critpath.update"
    expected_subti = "ralph altered the critpath status for some packages"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set([])

    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1357581512.006664,
        "topic": "org.fedoraproject.stg.pkgdb.critpath.update",
        "msg": {
            "package_listing_ids": [],
            "agent": "ralph",
            "critpath": True,
        },
    }


class TestPkgdbPackageRetire(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages on this topic when a package is retired.  For example:
    """
    expected_title = "pkgdb.package.retire"
    expected_subti = "ralph retired php-zmq (EL-6)!"
    expected_link = "https://admin.fedoraproject.org/pkgdb/acls/name/php-zmq"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set(['php-zmq'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['php-zmq/retire'])
    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1357583297.886945,
        "topic": "org.fedoraproject.stg.pkgdb.package.retire",
        "msg": {
            "package_listing": {
                "owner": "orphan",
                "package": {
                    "upstreamurl": None,
                    "name": "php-zmq",
                    "description": None,
                    "reviewurl": None,
                    "summary": "PHP 0MQ/zmq/zeromq extension"
                },
                "qacontact": None,
                "collection": {
                    "pendingurltemplate": None,
                    "name": "Fedora EPEL",
                    "publishurltemplate": None,
                    "version": "6",
                    "disttag": ".el6",
                    "branchname": "EL-6"
                },
                "specfile": None
            },
            "retirement": "retired",
            "agent": "ralph"
        }
    }


class TestPkgdbUserRemove(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    when a user is removed from a package ACL.
    """
    expected_title = "pkgdb.acl.user.remove"
    expected_subti = "ralph removed ralph from php-zmq (EL-6, F18)"
    expected_link = "https://admin.fedoraproject.org/pkgdb/acls/name/php-zmq"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set(['php-zmq'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['php-zmq/remove/ralph'])
    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1357583297.886945,
        "topic": "org.fedoraproject.stg.pkgdb.acl.user.remove",
        "msg": {
            "package_listings": [{
                "owner": "orphan",
                "package": {
                    "upstreamurl": None,
                    "name": "php-zmq",
                    "description": None,
                    "reviewurl": None,
                    "summary": "PHP 0MQ/zmq/zeromq extension"
                },
                "qacontact": None,
                "collection": {
                    "pendingurltemplate": None,
                    "name": "Fedora EPEL",
                    "publishurltemplate": None,
                    "version": "6",
                    "disttag": ".el6",
                    "branchname": "EL-6"
                },
                "specfile": None
            }, {
                "owner": "orphan",
                "package": {
                    "upstreamurl": None,
                    "name": "php-zmq",
                    "description": None,
                    "reviewurl": None,
                    "summary": "PHP 0MQ/zmq/zeromq extension"
                },
                "qacontact": None,
                "collection": {
                    "pendingurltemplate": None,
                    "name": "Fedora",
                    "publishurltemplate": None,
                    "version": "18",
                    "disttag": ".f18",
                    "branchname": "F18"
                },
                "specfile": None
            }],
            "collections": [
                # This actually has stuff in it in prod.
            ],
            "username": "ralph",
            "agent": "ralph",
        }
    }


class TestPkgdbBranchStart(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when branching starts.
    """
    expected_title = "pkgdb.branch.start"
    expected_subti = "ralph started a branch of F-19 from devel"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set()
    msg = {
        u'username': u'threebean',
        u'i': 1,
        u'timestamp': 1379606342.105066,
        u'msg_id': u'2013-0eaf6d98-6259-4e1c-a113-e2c9284a6082',
        u'topic':
        u'org.fedoraproject.dev.pkgdb.branch.start',
        u'msg': {
            u'collection_from': {
                u'pendingurltemplate': None,
                u'publishurltemplate': None,
                u'branchname': u'devel',
                u'name': u'Fedora',
                u'version': u'devel'
            },
            u'collection_to': {
                u'pendingurltemplate': None,
                u'publishurltemplate': None,
                u'branchname': u'F-19',
                u'name': u'Fedora',
                u'version': u'19'
            },
            u'agent': u'ralph',
        },
    }


class TestPkgdbBranchComplete(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when branching completes.
    """
    expected_title = "pkgdb.branch.complete"
    expected_subti = "ralph's branch of F-19 from devel completed"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set()
    msg = {
        u'username': u'threebean',
        u'i': 1,
        u'timestamp': 1379606342.105066,
        u'msg_id': u'2013-0eaf6d98-6259-4e1c-a113-e2c9284a6082',
        u'topic':
        u'org.fedoraproject.dev.pkgdb.branch.complete',
        u'msg': {
            u'collection_from': {
                u'pendingurltemplate': None,
                u'publishurltemplate': None,
                u'branchname': u'devel',
                u'name': u'Fedora',
                u'version': u'devel'
            },
            u'collection_to': {
                u'pendingurltemplate': None,
                u'publishurltemplate': None,
                u'branchname': u'F-19',
                u'name': u'Fedora',
                u'version': u'19'
            },
            u'agent': u'ralph',
        },
    }


class TestPkgdbCollectionNew(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when an admin creates a new collection.
    """
    expected_title = "pkgdb.collection.new"
    expected_subti = "ralph created a new collection for Fedora 19"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set()
    msg = {
        u'username': u'threebean',
        u'i': 3,
        u'timestamp': 1379607327.474346,
        u'msg_id': u'2013-68fd388e-60ca-4cf6-888d-b51161798496',
        u'topic': u'org.fedoraproject.dev.pkgdb.collection.new',
        u'msg': {
            u'collection': {
                u'pendingurltemplate': None,
                u'publishurltemplate': None,
                u'branchname': u'F-19',
                u'name': u'Fedora',
                u'version': u'19',
            },
            u'agent': u'ralph',
        }
    }


class TestPkgdbCollectionUpdate(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when an admin creates a new collection.
    """
    expected_title = "pkgdb.collection.update"
    expected_subti = ("ralph updated the following fields of the Fedora 18 "
                      "collection: name, version")
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = ("http://www.gravatar.com/avatar/"
                               "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
                               "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set()
    msg = {
        u'username': u'threebean',
        u'i': 27,
        u'timestamp': 1379607692.198447,
        u'msg_id': u'2013-478a321f-ddfc-4d4c-adeb-c777619da15a',
        u'topic': u'org.fedoraproject.dev.pkgdb.collection.update',
        u'msg': {
            u'fields': [
                u'name',
                u'version',
            ],
            u'collection': {
                u'pendingurltemplate': u'http://.....',
                u'publishurltemplate': u'http://.....',
                u'branchname': u'f18_b',
                u'name': u'Fedora',
                u'version': u'18'
            },
            u'agent': u'ralph',
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
from fedmsg_meta_fedora_infrastructure import BaseProcessor

from fasshim import gravatar_url

fs_prefix = "/srv/web/infra/ansible/"


def relative_playbook(playbook):
    """ Returns a tuple (controlled, playbook).

    - controlled is a boolean indicating whether or not we think that the
      playbook being run was checked in to our ansible git repo.
    - playbook is the relative file path of the playbook.
    """
    if playbook.startswith(fs_prefix):
        return True, playbook[len(fs_prefix):]
    else:
        return False, playbook.split('/')[-1]


class AnsibleProcessor(BaseProcessor):
    __name__ = "ansible"
    __description__ = "Fedora Infrastructure Ansible Runs"
    __link__ = "http://infrastructure.fedoraproject.org/cgit/ansible.git"
    __icon__ = "https://apps.fedoraproject.org/img/icons/ansible.png"
    __docs__ = \
        "https://fedoraproject.org/wiki/Infrastructure_ansible_migration"
    __obj__ = "Ansible Runs"

    def subtitle(self, msg, **config):
        user = msg['msg']['userid']
        controlled, playbook = relative_playbook(msg['msg']['playbook'])

        if 'ansible.playbook.start' in msg['topic']:
            tmpl = self._("{user} started an ansible run of {playbook}")
            return tmpl.format(user=user, playbook=playbook)
        elif 'ansible.playbook.complete' in msg['topic']:
            tmpl = self._("{user}'s {playbook} playbook run completed")
            return tmpl.format(user=user, playbook=playbook)
        return ""

    def secondary_icon(self, msg, **config):
        user = msg['msg']['userid']
        return gravatar_url(user)

    def link(self, msg, **config):
        base = "http://infrastructure.fedoraproject.org/cgit/ansible.git/tree/"
        controlled, playbook = relative_playbook(msg['msg']['playbook'])
        if not controlled:
            return None
        else:
            return base + playbook

    def usernames(self, msg, **config):
        return set([msg['msg']['userid']])

    def objects(self, msg, **config):
        controlled, playbook = relative_playbook(msg['msg']['playbook'])

        if not controlled:
            playbook = "uncontrolled-playbooks/" + playbook

        if 'results' in msg['msg']:
            return set([playbook] + [
                "inventory/" + host for host in msg['msg']['results'].keys()
            ])
        else:
            return set([playbook])



# This file is part of fedmsg.
# Copyright (C) 2013 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>

from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fasshim import gravatar_url


class CoprsProcessor(BaseProcessor):
    __name__ = "Copr"
    __description__ = "the Cool Other Package Repositories system"
    __link__ = "https://copr-fe.cloud.fedoraproject.org"
    __docs__ = "http://fedorahosted.org/copr"
    __obj__ = "Extra Repository Updates"
    # No copr icon yet..
    #__icon__ = ....

    def subtitle(self, msg, **config):

        user = msg['msg'].get('user')
        copr = msg['msg'].get('copr')
        chroot = msg['msg'].get('chroot')

        if 'copr.build.start' in msg['topic']:
            tmpl = self._("{user} started a new build of the {copr} copr")
        elif 'copr.build.end' in msg['topic']:
            tmpl = self._("{user}'s {copr} copr finished a build")
        elif 'copr.chroot.start' in msg['topic']:
            tmpl = self._("{user}'s {copr} copr started a new {chroot} chroot")
        elif 'copr.worker.create' in msg['topic']:
            tmpl = self._("a new worker was created")
        else:
            raise NotImplementedError()

        return tmpl.format(user=user, copr=copr, chroot=chroot)

    def link(self, msg, **config):
        user = msg['msg']['user']
        copr = msg['msg']['copr']
        chroot = msg['msg'].get('chroot', None)

        if 'chroot' in msg['topic']:
            tmpl = ("http://copr-be.cloud.fedoraproject.org/"
                    "results/{user}/{copr}/{chroot}/")
        elif 'build' in msg['topic']:
            tmpl = ("http://copr-fe.cloud.fedoraproject.org/"
                    "coprs/{user}/{copr}/")
        else:
            return None

        return tmpl.format(user=user, copr=copr, chroot=chroot)

    def secondary_icon(self, msg, **config):
        if 'user' in msg['msg']:
            return gravatar_url(msg['msg']['user'])

    def usernames(self, msg, **config):
        if 'user' in msg['msg']:
            return set([msg['msg']['user']])
        return set()

    def objects(self, msg, **config):
        items = ['coprs']

        if 'copr' in msg['msg']:
            items.append(msg['msg']['copr'])

        items.append('.'.join(msg['topic'].split('.')[-2:]))

        if 'chroot' in msg['topic']:
            items.append(msg['msg']['chroot'])

        return set(['/'.join(items)])



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fasshim import gravatar_url


class WikiProcessor(BaseProcessor):
    __name__ = "Wiki"
    __description__ = "the Fedora Wiki"
    __link__ = "https://fedoraproject.org/wiki"
    __docs__ = "https://fedoraproject.org/wiki"
    __obj__ = "Wiki Edits"
    __icon__ = "https://fedoraproject.org/w/skins/common/images/mediawiki.png"

    def subtitle(self, msg, **config):
        if 'wiki.article.edit' in msg['topic']:
            user = msg['msg']['user']
            title = msg['msg']['title']
            tmpl = self._('{user} made a wiki edit to "{title}"')
            return tmpl.format(user=user, title=title)
        elif 'wiki.upload.complete' in msg['topic']:
            user = msg['msg']['user_text']
            filename = msg['msg']['title']['mPrefixedText']
            description = msg['msg']['description'][:35]
            tmpl = self._(
                '{user} uploaded {filename} to the wiki: "{description}..."'
            )
            return tmpl.format(user=user, filename=filename,
                               description=description)

    def link(self, msg, **config):
        if 'wiki.article.edit' in msg['topic']:
            return msg['msg']['url']

    def secondary_icon(self, msg, **config):
        user = msg['msg'].get('user', msg['msg'].get('user_text', ''))
        return gravatar_url(user.lower())

    def usernames(self, msg, **config):
        user = msg['msg'].get('user', msg['msg'].get('user_text', ''))
        return set([user.lower()])

    def objects(self, msg, **config):
        if 'wiki.article.edit' in msg['topic']:
            return set([msg['msg']['title'].strip('/') + '-page'])
        elif 'wiki.upload.complete' in msg['topic']:
            return set([msg['msg']['url'][1:]])

        return set()



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#

try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup

import sys

f = open('README.rst')
long_description = f.read().strip()
long_description = long_description.split('split here', 1)[1]
f.close()

# Ridiculous as it may seem, we need to import multiprocessing and
# logging here in order to get tests to pass smoothly on python 2.7.
try:
    import multiprocessing
    import logging
except Exception:
    pass


install_requires = [
    'fedmsg',
    'python-fedora',
    'python-dateutil',
]
tests_require = [
    'nose',
]

if sys.version_info < (2, 7):
    install_requires.extend([
        'argparse',
        'ordereddict',
    ])
    tests_require.extend([
        'unittest2',
    ])

entry_points = {
    'fedmsg.meta': [
        "askbot=fedmsg_meta_fedora_infrastructure.askbot:AskbotProcessor",
        "bodhi=fedmsg_meta_fedora_infrastructure.bodhi:BodhiProcessor",
        "scm=fedmsg_meta_fedora_infrastructure.scm:SCMProcessor",
        "tagger=fedmsg_meta_fedora_infrastructure.tagger:TaggerProcessor",
        "planet=fedmsg_meta_fedora_infrastructure.planet:PlanetProcessor",
        "bot=fedmsg_meta_fedora_infrastructure.supybot:SupybotProcessor",
        "wiki=fedmsg_meta_fedora_infrastructure.mediawiki:WikiProcessor",
        "fas=fedmsg_meta_fedora_infrastructure.fas:FASProcessor",
        "comp=fedmsg_meta_fedora_infrastructure.compose:ComposeProcessor",
        "pkgdb=fedmsg_meta_fedora_infrastructure.pkgdb:PkgdbProcessor",
        "buildsys="
        "fedmsg_meta_fedora_infrastructure.buildsys:KojiProcessor",
        "trac=fedmsg_meta_fedora_infrastructure.trac:TracProcessor",
        "mailman=fedmsg_meta_fedora_infrastructure.mailman3:MailmanProcessor",
        "fedbadges=fedmsg_meta_fedora_infrastructure.badges:BadgesProcessor",
        "ansible=fedmsg_meta_fedora_infrastructure.ansible:AnsibleProcessor",
        "datanommer=fedmsg_meta_fedora_infrastructure.datanommer:"
        "DatanommerProcessor",
        "nuancier=fedmsg_meta_fedora_infrastructure.nuancier:"
        "NuancierProcessor",
        "fedocal=fedmsg_meta_fedora_infrastructure.fedocal:FedocalProcessor",
        "coprs=fedmsg_meta_fedora_infrastructure.coprs:CoprsProcessor",
        "cnucnuweb=fedmsg_meta_fedora_infrastructure.cnucnuweb:CnuCnuWebProcessor",
        "summershum=fedmsg_meta_fedora_infrastructure.summershum:SummerShumProcessor",
        "jenkins=fedmsg_meta_fedora_infrastructure.jenkins:JenkinsProcessor",
        "github=fedmsg_meta_fedora_infrastructure.github:GithubProcessor",
    ]
}

setup(
    name='fedmsg_meta_fedora_infrastructure',
    version='0.2.11',
    description=
    "fedmsg metadata providers for Fedora Infrastructure's deployment",
    long_description=long_description,
    author='Ralph Bean',
    author_email='rbean@redhat.com',
    url='http://github.com/ralphbean/fedmsg_meta_fedora_infrastructure/',
    license='LGPLv2+',
    install_requires=install_requires,
    tests_require=tests_require,
    test_suite='nose.collector',
    packages=[
        'fedmsg_meta_fedora_infrastructure',
        'fedmsg_meta_fedora_infrastructure.tests',
    ],
    include_package_data=True,
    zip_safe=False,
    entry_points=entry_points
)
